/*
 * parser.c -  parser for driver vkontakte of MSA program
 * This file is part of MSA program
 *
 * Copyright (C) 2009 - Sergey Zakharov
 *
 * MSA program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MSA program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MSA program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 */
#include "parser.h"

const char* gender[3] = {
    NO_MALE,
    FEMALE,
    MALE
};
const char* family_state[7] = {
    FAMILY_STATE_0,
    FAMILY_STATE_1, 
    FAMILY_STATE_2, 
    FAMILY_STATE_3, 
    FAMILY_STATE_4, 
    FAMILY_STATE_5, 
    FAMILY_STATE_6,  
};

const char* pol_view[9] = {
    POL_VIEW_0,
    POL_VIEW_1, 
    POL_VIEW_2, 
    POL_VIEW_3, 
    POL_VIEW_4, 
    POL_VIEW_5, 
    POL_VIEW_6, 
    POL_VIEW_7, 
    POL_VIEW_8 
};

/**
 * Parses json data, validate it
 *
 * @param json - data in json format
 * @return 0 - data valid,  -1, 1 - server response status  or -2, -3 if data not valid
 */
int check(char* json)
{
    g_debug("check START");
    int check_status = 0;
    struct json_object *buf = NULL;
    struct json_object *status = NULL;
    if(json != NULL) {
        status = json_tokener_parse(json);
        if(is_error(status))
            return -2;
        if(status != NULL) {
            if (json_object_get_type(status) == json_type_array) {
                json_object_put(status);
                return 0;
            }
            //Get server response status
            buf = json_object_object_get(status, "ok" );
            if(buf != NULL) {
                g_debug("read ok status = %d", json_object_get_int(buf));
		check_status = json_object_get_int(buf);
                json_object_put(status);
                return check_status;
            } else {
                json_object_put(status);	
                return 0;
            }
        }
    }
    return -3;
}
/**
 * Parses image data, form xml 
 *
 * @param doc - response
 * @param data - image
 * @return 0 on success or 1 otherwise.
 */

int parser_image(xmlDocPtr doc, char* data, char* url)
{
    xmlNodePtr rootNode, childNode;
    rootNode = xmlDocGetRootElement(doc);     //response 
    if (rootNode != NULL) {
        rootNode = rootNode->children;           //params

        childNode = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_IMG, BAD_CAST data);
        //g_free(data);
	//TODO
	//Save file on disk
	//insert name of file
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "Img");
        xmlSetProp(childNode, BAD_CAST "uri", BAD_CAST url);
        xmlAddChild(rootNode, childNode);
        //xmlDocDump(stdout, doc);
        return 0;
    }

    return 1;
}
/**
 * Parses json data, form xml profile
 *
 * @param doc - response
 * @param json - data in json format
 * @param cp - current profile
 * @return 0 on success or 1 otherwise.
 */
int parser_profile(xmlDocPtr doc, char* json, char* remixmid, gboolean image, struct ProfileData *cp)
{
    int i = 0, buf_serialized = 0;
    gchar* id = NULL;
    struct json_object* buf = NULL, *profile = NULL, *ht = NULL, *edu = NULL, *edu_item = NULL, *activity = NULL;
    struct json_object *ph = NULL, *hash = NULL, *rhash = NULL, *url = NULL, *aid = NULL;     
    xmlNodePtr rootNode, childNode, node, node2;

    rootNode = xpath_get_node("//Response/Params", doc);
	if(rootNode == NULL)
        return 1;

    profile = json_tokener_parse(json);
    if(profile != NULL){
        buf = json_object_object_get(profile, ID_KEY );
        if(buf == NULL)
            return 1;
        //PARSE FRIEND'S ID
        if (strcmp(remixmid, json_object_get_string(buf)) != 0){
            id = g_strconcat(PREFIX, json_object_get_string(buf), NULL);
            xmlSetProp(rootNode, BAD_CAST "id", BAD_CAST id);
            g_free(id);
	    }
        //PARSE FIRST NAME
        buf = json_object_object_get(profile, FIRST_NAME_KEY );
        if(buf == NULL)
            return 1;
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST FIRST_NAME);
        node = xmlAddChild(rootNode, node);

        //PARSE LAST NAME
        buf = json_object_object_get(profile, LAST_NAME_KEY );
        if(buf == NULL)
            return 1;
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST LAST_NAME);
        node = xmlAddChild(rootNode, node);

        //PARSE ACTIVITY
        buf = json_object_object_get(profile, ACTIVITY_KEY );
        if(buf != NULL){
            activity = json_object_object_get(buf, "5" );
            if(activity != NULL){
                node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(activity));
                xmlSetProp(node, BAD_CAST NAME, BAD_CAST ACTIVITY);
                node = xmlAddChild(rootNode, node);
            }
        }
        //PARSE IMAGE
        buf = json_object_object_get(profile, IMG_KEY );
        if(buf == NULL)
            return 1;
        char* img;
        if (image){
            if(http_get(json_object_get_string(buf), info_proxy, info_port, NULL, &img) == 200) {
                node = xmlNewDocNode(doc, NULL,  BAD_CAST "img", BAD_CAST img);
                xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Img");
                xmlSetProp(node, BAD_CAST "uri", BAD_CAST json_object_get_string(buf));
                node = xmlAddChild(rootNode, node);
                g_debug("set image");
	        free(img);
            }
        } else {
            node = xmlNewDocNode(doc, NULL,  BAD_CAST "img", NULL);
            xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Img");
            xmlSetProp(node, BAD_CAST "uri", BAD_CAST json_object_get_string(buf));
            node = xmlAddChild(rootNode, node);
            g_debug("set image");
        }


        ht = json_object_object_get(profile, HABITATION_KEY);
        if(ht == NULL)
            return 1;

        //PARSE PHOTO UPLOADS PARAMS
        ph = json_object_object_get(profile, PH_KEY );

        hash = json_object_object_get(ph, HASH_KEY );
        cp->hash = g_strdup(json_object_get_string(hash));

        rhash = json_object_object_get(ph, RHASH_KEY );
        cp->rhash = g_strdup(json_object_get_string(rhash));

        url = json_object_object_get(ph, URL_KEY );
        cp->upload_url = g_strdup(json_object_get_string(url));

        aid = json_object_object_get(ph, AID_KEY );
        cp->aid = g_strdup(json_object_get_string(aid));

        //PARSE CITY
        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "City");
        childNode = xmlAddChild(rootNode, childNode);

        buf = json_object_object_get(ht, CITY_ID_KEY);
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "CityId");
        node = xmlAddChild(childNode, node);
	
        buf = json_object_object_get(ht, CITY_KEY);
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST CITY_NAME);
        node = xmlAddChild(childNode, node);
	
        //PARSE COUNTRY
        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "Country");
        childNode = xmlAddChild(rootNode, childNode);

        buf = json_object_object_get(ht, COUNTRY_ID_KEY);
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "CountryId");
        node = xmlAddChild(childNode, node);
	
        buf = json_object_object_get(ht, COUNTRY_KEY);
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST COUNTRY_NAME);
        node = xmlAddChild(childNode, node);


        struct tm tm_date;
        memset(&tm_date, 0, sizeof(struct tm));

        buf = json_object_object_get(profile, BIRTH_DAY_KEY );
        g_debug("%s:%d: day=%s", __FILE__, __LINE__, json_object_get_string(buf));
        tm_date.tm_mday = json_object_get_int(buf);

        buf = json_object_object_get(profile, BIRTH_MONTH_KEY);
        g_debug("%s:%d: month=%s", __FILE__, __LINE__, json_object_get_string(buf));
        tm_date.tm_mon = json_object_get_int(buf) - 1;
	
        buf = json_object_object_get(profile, BIRTH_YEAR_KEY);
        g_debug("%s:%d: year=%s", __FILE__, __LINE__, json_object_get_string(buf));
        tm_date.tm_year = json_object_get_int(buf) - 1900;
	
        char date[20] = {0};
        if (tm_date.tm_year > 0 && tm_date.tm_mon > -1 &&  tm_date.tm_mday > 0) {
            //user set full date
            strftime(date, 19, "%d.%m.%Y", &tm_date);
        } else {
               if (tm_date.tm_mon > -1 &&  tm_date.tm_mday > 0){
                   //user set only day and month
                   strftime(date, 19, "%d.%m", &tm_date);
               } 
               if (tm_date.tm_year > 0 && tm_date.tm_mon > -1){
                   //user set only year and month
                   strftime(date, 19, "%m.%Y", &tm_date);
               }
               if (tm_date.tm_year > 0 && tm_date.tm_mon == -1)
               {
                   //user set only year
                   strftime(date, 19, "%Y", &tm_date);
               }
        }
                
        g_debug("date = %s", date);

        node = xmlNewDocNode(doc, NULL, BAD_CAST "date", BAD_CAST date);
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST BIRTHDAY_NAME);
        node = xmlAddChild(rootNode, node);
	
        //PARSE GENDER
        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_SELECT, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST GENDER_NAME);
        childNode = xmlAddChild(rootNode, childNode);

        buf = json_object_object_get(profile, MALE_KEY );
        int sx_serialized = json_object_get_int(buf);
	
        for (i = 0; i < 3; i++) {
            node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST gender[i]);
            if (sx_serialized == i)
                xmlSetProp(node, BAD_CAST "selected", BAD_CAST "1");
            else
                xmlSetProp(node, BAD_CAST "selected", BAD_CAST "0");
            node = xmlAddChild(childNode, node);	
        }

        //PARSE FAMILY STATE
        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_SELECT, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST FAMILY_STATE_NAME);
        childNode = xmlAddChild(rootNode, childNode);


        buf = json_object_object_get(profile, FAMILY_KEY );
        int family_serialized = json_object_get_int(buf);
        //json_object_put(buf);
	
        for (i = 0; i < 7; i++) {
            node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST family_state[i]);
            if (family_serialized == i)
                xmlSetProp(node, BAD_CAST "selected", BAD_CAST "1");
            else
                xmlSetProp(node, BAD_CAST "selected", BAD_CAST "0");
            node = xmlAddChild(childNode, node);
        }

        //PARSE POLITICAL VIEW
        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_SELECT, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST POLITICAL_VIEW_NAME);
        childNode = xmlAddChild(rootNode, childNode);

        buf = json_object_object_get(profile, POL_VIEW_KEY );
        int pol_view_serialized = json_object_get_int(buf);
	
        for (i = 0; i < 9; i++) {
            node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST pol_view[i]);
            if (pol_view_serialized == i)
                xmlSetProp(node, BAD_CAST "selected", BAD_CAST "1");
            else
                xmlSetProp(node, BAD_CAST "selected", BAD_CAST "0");
            node = xmlAddChild(childNode, node);	
        }

        //PARSE EDUCATION
        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_ARRAY, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "EducationList");
        childNode = xmlAddChild(rootNode, childNode);

        int length = 0;
        edu = json_object_object_get(profile, EDUCATION_KEY );
        if(edu != NULL){
            length=json_object_array_length(edu);

            for(i = 0; i < length; i++)
            {
                edu_item = json_object_array_get_idx(edu, i );
       
                buf = json_object_array_get_idx(edu_item, 2 );
                buf_serialized = json_object_get_int(buf);
                if (buf_serialized < 21 || buf_serialized == 100 ) {
                    node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRUCT, NULL );
                    xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Education");
                    node = xmlAddChild(childNode, node);
                    node2 = NULL;
                    if (buf_serialized < 21) {
                        node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST "School" );
                        node2 = xmlAddChild(node, node2);
                    } else { 
                        if (buf_serialized == 100) {
                            node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST "High school");
                            node2 = xmlAddChild(node, node2);
                        } 
                    }
                    xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "Type");

                    buf = json_object_array_get_idx(edu_item, 3 );		 
                    node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
                    node2 = xmlAddChild(node, node2);
                    xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "Name");
	
                    buf = json_object_array_get_idx(edu_item, 4 );
                    node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
                    node2 = xmlAddChild(node, node2);
                    xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "DateComplete");
	        }

            }

	    
	}
        json_object_put(profile);
	g_debug("free profile");
        return 0;
    }

    return 1;
}


/**
 * Parses json data, form xml profile
 *
 * @param doc - response
 * @param json - data in json format
 * @param cp - current profile
 * @return 0 on success or 1 otherwise.
 */
int parser_profile_albums(xmlDocPtr doc, char* json, char* remixmid, struct ProfileData *cp)
{
    struct json_object *profile = NULL;
    struct json_object *ph = NULL, *hash = NULL, *rhash = NULL, *url = NULL, *aid = NULL;     
    xmlNodePtr rootNode, childNode, node;

    rootNode = xpath_get_node("//Response/Params", doc);
	if(rootNode == NULL)
        return 1;

    profile = json_tokener_parse(json);
    if(profile != NULL){

        //PARSE PHOTO UPLOADS PARAMS
        ph = json_object_object_get(profile, PH_KEY );

        hash = json_object_object_get(ph, HASH_KEY );
        cp->hash = g_strdup(json_object_get_string(hash));

        rhash = json_object_object_get(ph, RHASH_KEY );
        cp->rhash = g_strdup(json_object_get_string(rhash));

        url = json_object_object_get(ph, URL_KEY );
        cp->upload_url = g_strdup(json_object_get_string(url));

        aid = json_object_object_get(ph, AID_KEY );
        cp->aid = g_strdup(json_object_get_string(aid));

        childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_ARRAY, NULL );
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "albumList");
        childNode = xmlAddChild(rootNode, childNode);

        node = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "album");
        xmlSetProp(node, BAD_CAST "id", BAD_CAST cp->aid);
        node = xmlAddChild(childNode, node);
	
        json_object_put(profile);
        g_debug("free profile");
        return 0;
    }

    return 1;
}

/**
 * Parses json data, form xml friends list
 *
 * @param doc - response
 * @param json - data in json format
 * @return number of friends
 */
int parser_friends(char *json, xmlDocPtr doc, int counter)
{
    int i=0;
    int length = 0;
    int friend_count = 0;
    struct json_object *friend, *friends, *id, *name, *img, *status, *n, *fr, *container;
    char** friend_name;
    char* friend_surname;
    char* tmp;
    xmlNodePtr rootNode, childNode, node,node2;

    rootNode = xpath_get_node("//Response/Params", doc);
	if(rootNode == NULL)
        return 0;
    childNode = xpath_get_node("//Response/Params/array", doc);

    if (childNode == NULL) {
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_ARRAY,  NULL);
        childNode = xmlAddChild(rootNode, node);
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "contactList");
        tmp = g_strdup_printf("%d", counter);
        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST tmp);
        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);
        g_free(tmp);
    }

    container = json_tokener_parse(json);
    fr = json_object_object_get(container,"friends");
    n = json_object_object_get(fr,"n");
    friend_count = json_object_get_int(n);
    tmp = g_strdup_printf("%d", friend_count);
    xmlSetProp(childNode, BAD_CAST "quantity", BAD_CAST tmp);
    g_free(tmp);

    friends = json_object_object_get(fr,"d");
    length = json_object_array_length(friends);
    for(i = 0; i < length; i++) {
        friend = json_object_array_get_idx(friends, i );
        
        if (friend == NULL) {
            length = i;
            break;
            //xmlSetProp(childNode, "to", g_strdup_printf("%d", i + atoi(xmlGetProp(childNode,"to")) ) );
           //return i;
        }
        id = json_object_array_get_idx(friend, 0 );
        if (id == NULL) {
            length = i;
            break;
        }
        node = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "contact");
        tmp = g_strconcat(PREFIX, json_object_get_string(id), NULL);
        xmlSetProp(node, BAD_CAST "id", BAD_CAST tmp);
        node = xmlAddChild(childNode, node);
        free(tmp);

        name = json_object_array_get_idx(friend, 1 );
        if(name == NULL){
            xmlUnlinkNode(node);
            length = i;
            break;
        }
        
        friend_name = g_strsplit(json_object_get_string(name), " ",2);
        //json_object_put(name);
        if( friend_name[1] == NULL)
            friend_surname = g_strdup(" ");
        else
            friend_surname = g_strdup(friend_name[1]);

        tmp = g_strconcat(friend_surname, " ", friend_name[0], NULL);
        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRING, BAD_CAST tmp);
        free(tmp); 
        free(friend_surname);

        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "FriendName");
        node2 = xmlAddChild(node, node2);
        g_strfreev (friend_name);

        status = json_object_array_get_idx(friend, 3 );
        if(status == NULL){
            xmlUnlinkNode(node);
            length = i;
            break;
        }	
        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(status));
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "FriendStatus");
        node2 = xmlAddChild(node, node2);
        //json_object_put(status);

        img = json_object_array_get_idx(friend, 2 );
        if(img == NULL){
            xmlUnlinkNode(node);
            length = i;
            break;
        }
	
        g_debug("IMG=%s",json_object_get_string(img));

        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST "img", NULL);
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "FriendImg");
        if (strcmp(json_object_get_string(img), "0") != 0) {
            xmlSetProp(node2, BAD_CAST "uri", BAD_CAST json_object_get_string(img));
        }
        node2 = xmlAddChild(node, node2);
        //json_object_put(img);
        //json_object_put(friend);
    }
    json_object_put(friends);
    xmlChar* len_str = xmlGetProp(childNode, BAD_CAST "to");
    tmp = g_strdup_printf("%d", length + atoi((gchar *)len_str));
    xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);

    g_free(tmp);
    xmlFree(len_str);

    return length;
}

/**
 * Parses json data, form xml list of messages
 * @param doc - response
 * @param json - data in json format
 * @return number of messages
 */
int parser_messages(char *json, xmlDocPtr doc, gchar *message_id, int counter, int max_count)
{
    g_debug("parser_messages START");
    //g_debug("Messages JSON: \n %s", json);
    int i = 0;
    int length = 0;
    int message_count;
    struct json_object *mess, *message, *messages, *id, *sender, *time, *text, *status, *n;
    char* tmp_str; 
    xmlNodePtr rootNode = NULL, childNode = NULL, node = NULL, node2 = NULL;

    rootNode = xpath_get_node("//Response/Params", doc);
	if(rootNode == NULL)
        return 0;
    childNode = xpath_get_node("//Response/Params/array", doc);

    if (childNode == NULL) {
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_ARRAY,  NULL);
        childNode = xmlAddChild(rootNode, node);
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "messagetList");
        tmp_str = g_strdup_printf("%d", counter);
        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST tmp_str);
        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp_str);
        g_free(tmp_str);
    }

    mess = json_tokener_parse(json);
    n = json_object_object_get(mess,"n");
    message_count = json_object_get_int(n);
    tmp_str = g_strdup_printf("%d", message_count);
    xmlSetProp(childNode, BAD_CAST "quantity", BAD_CAST tmp_str);
    g_free(tmp_str);

    messages = json_object_object_get(mess,"d");
    length = json_object_array_length(messages);
    g_debug("Start parse");
    if (length > (max_count - counter) || length > PAGE_SIZE)
        length--;
    for(i = 0; i < length; i++) {

        message = json_object_array_get_idx(messages, i );

        if (message == NULL) {
    	    gchar *msg = g_strdup_printf("%d", i + atoi((gchar *)xmlGetProp(childNode, BAD_CAST "to")));
            xmlSetProp(childNode, BAD_CAST "to",  BAD_CAST msg);
            g_free(msg);
            return i;
        }

        id = json_object_array_get_idx(message, 0);
        node = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "message");
        if(id == NULL){
            break;
        }
        if(g_strcmp0(json_object_get_string(id), message_id) == 0)
            break;
        tmp_str = g_strconcat(PREFIX, json_object_get_string(id), NULL);
        xmlSetProp(node, BAD_CAST "id", BAD_CAST tmp_str);
        node = xmlAddChild(childNode, node);
	    g_free(tmp_str);

        sender = json_object_array_get_idx(message, 3);
        if(sender == NULL)
            break;
        
        id = json_object_array_get_idx(sender, 0 );
        if(id == NULL)
            break;
        tmp_str = g_strconcat(PREFIX, json_object_get_string(id), NULL);
        node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST tmp_str);
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "SenderId");
        node2 = xmlAddChild(node, node2);
        g_free(tmp_str);

        char** name_list;
        id = json_object_array_get_idx(sender, 1 );
        //if(id == NULL)
           // break;
        if (json_object_get_string(id) != NULL) {
            name_list = g_strsplit(json_object_get_string(id), "\t", 2);
	    tmp_str = g_strconcat(name_list[0], " ", name_list[1], NULL);
            node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST tmp_str);
	    g_free(tmp_str);
            g_strfreev(name_list);
        } else {
                node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, NULL);
        }
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "SenderName");
        node2 = xmlAddChild(node, node2);

        sender = json_object_array_get_idx(message, 4);
        if(sender == NULL)
             break;
        id = json_object_array_get_idx(sender, 0 );
        if(id == NULL)
            break;
        tmp_str = g_strconcat(PREFIX, json_object_get_string(id), NULL);
        node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST tmp_str);
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "RecipientId");
        node2 = xmlAddChild(node, node2);
        g_free(tmp_str);

        id = json_object_array_get_idx(sender, 1 );
        //if(id == NULL)
            //break;
        if (json_object_get_string(id) != NULL) {
            name_list = g_strsplit(json_object_get_string(id), "\t", 2);
            tmp_str = g_strconcat(name_list[0], " ", name_list[1], NULL);
            node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST tmp_str);
            g_free(tmp_str);
            g_strfreev(name_list);
        } else {
                node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, NULL);
        }
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "RecipientName");
        node2 = xmlAddChild(node, node2);

        //g_debug("parse_time");
        time = json_object_array_get_idx(message,1);
        if(time == NULL)
            break;
        unsigned long int t;
        t = json_object_get_int(time);

        tmp_str = g_strdup_printf("%lu", t);
        node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST tmp_str);
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "Time");
        node2 = xmlAddChild(node, node2);
        g_free(tmp_str);

        //g_debug("parse text");
        text = json_object_array_get_idx(message, 2);
        if(text == NULL)
            break;
        char* mess_text = json_object_get_string(text);
        char* tmp;
        while((tmp = strstr(mess_text, "<br>")) != NULL)
        {
            int k = 0;
            for(; k < 3; k++)
                *(tmp + k) = ' ';
            *(tmp + k) = '\n';
        }

        char* temp;
        char** list;

        list = g_strsplit(mess_text,"\"",3);
        temp = g_strdup(list[1]);
        g_strfreev (list);

        node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST temp);
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "Text");
        node2 = xmlAddChild(node, node2);
        g_free(temp);
        
        //g_debug("parse read_status");
        status = json_object_array_get_idx(message,5);
        node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(status));
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "Status");
        node2 = xmlAddChild(node, node2);
    }
    if(i != length){
        if(node != NULL){
            xmlUnlinkNode(node);
            xmlFreeNode(node);
        }
        length = i;
    }
    json_object_put(mess);
    xmlChar* len_str = xmlGetProp(childNode, BAD_CAST "to");
    tmp_str = g_strdup_printf("%d", length + atoi((gchar *)len_str));
    xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp_str );

    g_free(tmp_str);
    xmlFree(len_str);

    g_debug("messages_length = %d", length);
    g_debug("parser_messages END");
    return length;
}

/**
 * Parses json data, form xml
 * @param json - data in json format
 * @return number of new messages
 */
int parser_new_messages(char* json)
{
    struct json_object *buf = NULL;
    struct json_object *num;
    int nm = 0;
    if(json != NULL) {
        buf = json_tokener_parse(json);
        if(buf != NULL){
            num = json_object_object_get(buf, "nm" );
            if(num != NULL) {
                nm = json_object_get_int(num);
		json_object_put(buf);
                return nm;
            } else {
                json_object_put(buf);
                return 0;
            }
	}
    }
    return 0;
}


/**
 * Parses json data, form xml photos list
 *
 * @param doc - response
 * @param json - data in json format
 * @return number of photos
 */
int parser_photos(char *json, xmlDocPtr doc, int counter)
{
    int i=0;
    int length = 0;
    int photo_count = 0;
    struct json_object *photo, *photos, *id, *url_small, *url_big;
    char* tmp;
    xmlNodePtr rootNode, childNode, node,node2;

    rootNode = xpath_get_node("//Response/Params", doc);
	if(rootNode == NULL)
        return 0;
    childNode = xpath_get_node("//Response/Params/array", doc);

    if (childNode == NULL) {
        node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_ARRAY,  NULL);
        childNode = xmlAddChild(rootNode, node);
        xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "photosList");
        tmp = g_strdup_printf("%d", counter);
        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST tmp);
        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);
        g_free(tmp);
    }

    photos = json_tokener_parse(json);

    length = json_object_array_length(photos);
    for(i = 0; i < length; i++) {
        photo = json_object_array_get_idx(photos, i );
        
        if (photo == NULL) {
            length = i;
            break;
        }
        id = json_object_array_get_idx(photo, 0 );
        if (id == NULL) {
            length = i;
            break;
        }
        node = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
        xmlSetProp(node, BAD_CAST NAME, BAD_CAST "photo");
        tmp = g_strconcat(PREFIX, json_object_get_string(id), NULL);
        xmlSetProp(node, BAD_CAST "id", BAD_CAST tmp);
        node = xmlAddChild(childNode, node);
        free(tmp);

        url_small = json_object_array_get_idx(photo, 1 );
        if(url_small == NULL){
            xmlUnlinkNode(node);
            length = i;
            break;
        }
        
        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(url_small));
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "urlSmall");
        node2 = xmlAddChild(node, node2);


        url_big = json_object_array_get_idx(photo, 2 );
        if(url_big == NULL){
            xmlUnlinkNode(node);
            length = i;
            break;
        }	
        node2 = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(url_big));
        xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "urlBig");
        node2 = xmlAddChild(node, node2);

    }
    json_object_put(photos);
    xmlChar* len_str = xmlGetProp(childNode, BAD_CAST "to");
    tmp = g_strdup_printf("%d", length + atoi((gchar *)len_str));
    xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);

    g_free(tmp);
    xmlFree(len_str);

    return length;
}
